/* 
 *  COPYRIGHT (c) 1989-2007.
 *  On-Line Applications Research Corporation (OAR).
 *
 *  The license and distribution terms for this file may be found in
 *  the file LICENSE in this distribution or at
 *  http://www.rtems.com/license/LICENSE.
 *
 *  $Id: init.c,v 1.6 2007/08/17 20:24:36 joel Exp $
 */

#include <bsp.h>

#include <assert.h>
#include <pthread.h>
#include <stdlib.h>

extern rtems_configuration_table  BSP_Configuration;

/*
 *  Using GNAT's Distributed Systems Annex support requires
 *  each node in the system to have different pid's.  In a
 *  single CPU RTEMS system, it is always one.  This lets the
 *  user override the RTEMS default.
 */
#ifdef GNAT_PID
  #include <unistd.h>

  pid_t getpid()
  {
    return GNAT_PID;
  }
#endif

/*
 *  By having the POSIX_Init thread create a second thread just
 *  to invoke gnat_main, we can override all default attributes
 *  of the "Ada environment task".  Otherwise, we would be
 *  stuck with the defaults set by RTEMS.
 */
 
void *start_gnat_main( void * argument )
{
  extern int gnat_main ( int argc, char **argv, char **envp );

  (void) gnat_main ( 0, 0, 0 );

  exit( 0 );

  return 0;
}

#ifndef GNAT_MAIN_STACKSPACE
  #define GNAT_MAIN_STACKSPACE 0
#endif

void *POSIX_Init( void *argument )
{
  pthread_t       thread_id;
  pthread_attr_t  attr;
  size_t          stacksize;
  int             status;
  extern  size_t  _ada_pthread_minimum_stack_size();

  status = pthread_attr_init( &attr );
  assert( !status );

  stacksize = GNAT_MAIN_STACKSPACE * 1024;
  if ( stacksize < _ada_pthread_minimum_stack_size() )
    stacksize = _ada_pthread_minimum_stack_size();

  status = pthread_attr_setstacksize( &attr, stacksize );
  assert( !status );

  status = pthread_create( &thread_id, &attr, start_gnat_main, NULL );
  assert( !status );

  pthread_exit( 0 );
  return 0;
}

/* configuration information */

/* Standard output and a clock tick so time passes */
#define CONFIGURE_APPLICATION_NEEDS_CONSOLE_DRIVER
#define CONFIGURE_APPLICATION_NEEDS_CLOCK_DRIVER

/* We want a clock tick every millisecond */
#define CONFIGURE_MICROSECONDS_PER_TICK RTEMS_MILLISECONDS_TO_MICROSECONDS(1)

/* The initialization task is a POSIX Initialization thread with default attributes */
#define CONFIGURE_POSIX_INIT_THREAD_TABLE

/* We are using GNAT/RTEMS with a maximum of 20 Ada tasks and no fake Ada tasks. */
/* A fake Ada task is a task created outside the Ada run-time that calls into Ada. */
#define CONFIGURE_GNAT_RTEMS
#define CONFIGURE_MAXIMUM_ADA_TASKS      20
#define CONFIGURE_MAXIMUM_FAKE_ADA_TASKS 0

/* Account for any extra task stack size */
#define CONFIGURE_MEMORY_OVERHEAD        (GNAT_MAIN_STACKSPACE)

#define CONFIGURE_INIT

#include <rtems/confdefs.h>
